// PaulaListPreviewPrint.cpp : implementation file
//

#include "stdafx.h"
//#include "Paula.h"
#include "PaulaListPreviewPrint.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////
// Cache entry class

class CPrintCacheEntry
{
public:
	CPrintCacheEntry() 
		: m_hIcon(NULL)
		{};

public:
	CString m_strText;
	HICON m_hIcon;	
};

////////////////////////////////////////
// Print Cache class

class CPrintCache
{
public:

	CPrintCache();
	CPrintCache(UINT xSize,UINT ySize);
	~CPrintCache();

	void SetSize(UINT xSize,UINT ySize);

	void Put(UINT x, UINT y, LPCTSTR lpszText, HICON hIcon);
	CPrintCacheEntry& Get(UINT x, UINT y) const;

	void Clear();

	UINT GetXSize() const { return m_xSize; };
	UINT GetYSize() const { return m_ySize; };
	int GetValidLines() const { return m_nValidLines; };

	UINT m_nPrintPage;
	int m_nValidLines;

protected:

	CPrintCacheEntry** m_pData;
	UINT m_xSize;
	UINT m_ySize;
};

CPrintCache::CPrintCache() 
{
	m_pData = NULL;
	m_nPrintPage = 0;
	m_xSize = 0;
	m_ySize = 0;
	m_nValidLines = 0;
};

CPrintCache::CPrintCache(UINT xSize,UINT ySize)
{
	m_pData = NULL;
	m_nPrintPage = 0;
	m_xSize = 0;
	m_ySize = 0;
	m_nValidLines = 0;

	SetSize( xSize, ySize);
}

CPrintCache::~CPrintCache()
{
	Clear();
}

void CPrintCache::Clear()
{
	if ( NULL != m_pData )		
	{
		for ( UINT i = 0 ; i < m_xSize ; i++ )
		{
			delete[] (CPrintCacheEntry*)m_pData[i];
		}

		delete[] m_pData;
		m_pData = NULL;
	}
	m_nValidLines = 0;
}

void CPrintCache::SetSize(UINT xSize,UINT ySize)
{
	Clear();

	m_xSize = xSize;
	m_ySize = ySize;

	m_pData = new CPrintCacheEntry*[xSize];

	for ( UINT i = 0 ; i < xSize ; i++ )
	{
		m_pData[i] = new CPrintCacheEntry[ySize];
	}
}

inline void CPrintCache::Put(UINT x, UINT y, LPCTSTR lpszText, HICON hIcon )
{
	ASSERT(x < m_xSize && y < m_ySize);
	ASSERT( lpszText == NULL || AfxIsValidString( lpszText ) );

	m_pData[x][y].m_strText = lpszText;
	m_pData[x][y].m_hIcon = hIcon;

}

inline CPrintCacheEntry& CPrintCache::Get(UINT x, UINT y) const
{
	ASSERT(x < m_xSize && y < m_ySize);
	return m_pData[x][y];
}

/////////////////////////////////////////////////////////////////////////////
// CPaulaListPreviewPrint

static TCHAR szFont[] = _T("\\Font");
static TCHAR szHeight[] = _T("Height");
static TCHAR szWeight[] = _T("Weight");
static TCHAR szItalic[] = _T("Italic");
static TCHAR szUnderline[] = _T("Underline");
static TCHAR szPitchAndFamily[] = _T("PitchAndFamily");
static TCHAR szCharSet[] = _T("CharSet");
static TCHAR szFaceName[] = _T("FaceName");
static TCHAR szMargin[] = _T("\\Margin");
static TCHAR szLeft[] = _T("left");
static TCHAR szTop[] = _T("top");
static TCHAR szRight[] = _T("right");
static TCHAR szBottom[] = _T("bottom");


IMPLEMENT_DYNAMIC(CPaulaListPreviewPrint, CView)

static CDocument _DummyDoc;


// default font : Arial, 10
LOGFONT CPaulaListPreviewPrint::s_lf = 
{
  0xFFFFFFF3, 			// lfHeight
  0,								// lfWidth
  0,								// lfEscapement
  0,        			  // lfOrientation
  400,						  // lfWeight
  0,          			// lfItalic
  0,          			// lfUnderline
  0,								// lfStrikeOut
  DEFAULT_CHARSET,	// lfCharSet
  0,								// lfOutPrecision
  0,                // lfClipPrecision
  0,								// lfQuality
  0,								// lfPitchAndFamily
  _T("Arial")
};

// default margins
CRect CPaulaListPreviewPrint::s_rcMargins(2500, 2500, 2500, 2500);

CPaulaListPreviewPrint::CPaulaListPreviewPrint()
	: m_pPreviewState(NULL),
		m_hWndModelessDlg(NULL),
		m_hWndListCtrl(NULL),
		m_pPrintCache1(NULL),
		m_pPrintCache2(NULL)
{

	m_pDocument = &_DummyDoc;
	_DummyDoc.m_bAutoDelete = FALSE;
	m_iLRUCache = 0;
}


CPaulaListPreviewPrint::~CPaulaListPreviewPrint()
{
	m_pDocument = NULL; // Otherwise the framework will try
											// to detach 'this' from the document.

	//WriteProfileSettings();
	CleanUp();
}

void CPaulaListPreviewPrint::CleanUp()
{
	// Caches

	if ( NULL != m_pPrintCache1 )
	{
		delete m_pPrintCache1;
		m_pPrintCache1 = NULL;
	}	

	if ( NULL != m_pPrintCache2 )
	{
		delete m_pPrintCache2;
		m_pPrintCache2 = NULL;
	}	

	// Icons 

	int nSize = m_arIcons.GetSize();
	for ( int i = 0 ; i < nSize ; i++ )
	{
		HICON hIcon = m_arIcons[i];
		if ( NULL != hIcon )
		{
			::DestroyIcon( hIcon );
		}
	}
	m_arIcons.RemoveAll();

	// Columns info

	m_PrintColumns.RemoveAll();

	// Font 

	if ( (HFONT)m_Font )
	{
		m_Font.DeleteObject();
	}

	m_arStartItemForThisPage.RemoveAll();
}

void CPaulaListPreviewPrint::SetPrintSourceCtrl(CListCtrl& listCtrl)
{
	ASSERT_VALID(this);
	m_hWndListCtrl = listCtrl.GetSafeHwnd();
}

inline CListCtrl& CPaulaListPreviewPrint::GetListCtrl()
{
#ifdef _DEBUG
	if ( !::IsWindow(m_hWndListCtrl))
	{
		TRACE0("Error : CPaulaListPreviewPrint::GetListCtrl : no source list control.\n");
		TRACE0("Did you remember to call SetPrintSourceCtrl?\n");
		ASSERT(FALSE);
	}
#endif

	CListCtrl* pListCtrl = (CListCtrl*)CListCtrl::FromHandle(m_hWndListCtrl);

	ASSERT( pListCtrl->IsKindOf(RUNTIME_CLASS(CListCtrl)) || 
					pListCtrl->IsKindOf(RUNTIME_CLASS(CListView)) );

	return (*pListCtrl);
}

//
// String to be printed in the header
//
void CPaulaListPreviewPrint::SetHeaderText(LPCTSTR lpszHeader)
{
	ASSERT_VALID(this);
	m_strHeader = lpszHeader;
}

//
// Text to be printed on the cover page
//
void CPaulaListPreviewPrint::SetCoverPageText(LPCTSTR lpszCoverPageText)
{
	ASSERT_VALID(this);
	m_strCoverPage = lpszCoverPageText;
}

int CPaulaListPreviewPrint::GetListColumnCount()
{
	ASSERT(::IsWindow(m_hWndListCtrl));
	//
	// Get the column count. List control provides no way 
	// to accomlish it so we get it from embedded header control.
	//

	HWND hWndHeader = ListView_GetHeader(m_hWndListCtrl);
	ASSERT( NULL != hWndHeader );
	return Header_GetItemCount(hWndHeader);

}

void CPaulaListPreviewPrint::DoFontDialog()
{
	CPrintDialog dlgPrint(FALSE);

	if ( !AfxGetApp()->GetPrinterDeviceDefaults(&dlgPrint.m_pd) )
	{
		return;
	}

	HDC hdcPrint = dlgPrint.CreatePrinterDC();
	if (hdcPrint == NULL)
	{
		return;
	}

	CDC dcPrinter;
	dcPrinter.Attach(hdcPrint);

	CFontDialog dlg(&s_lf, CF_PRINTERFONTS, &dcPrinter);
	dlg.DoModal();
}

BOOL CPaulaListPreviewPrint::CreatePrintFont(CDC* pDC, CFont& font, LOGFONT logFont)
{	
	LOGFONT lf = logFont;
	LOGFONT lfSys;

	VERIFY(::GetObject(::GetStockObject(SYSTEM_FONT), sizeof(lfSys),
		&lfSys));

	if ( 0 != lstrcmpi((LPCTSTR)lf.lfFaceName, (LPCTSTR)lfSys.lfFaceName))
	{

		// Map to printer font metrics

		HDC hDCFrom = ::GetDC(NULL);

		lf.lfHeight = ::MulDiv(lf.lfHeight, pDC->GetDeviceCaps(LOGPIXELSY),
			::GetDeviceCaps(hDCFrom, LOGPIXELSY));
		lf.lfWidth = ::MulDiv(lf.lfWidth, pDC->GetDeviceCaps(LOGPIXELSX),
			::GetDeviceCaps(hDCFrom, LOGPIXELSX));

		::ReleaseDC(NULL, hDCFrom);
	}
		// Create font, if it fails we just use the printer's default.

	if ( (HFONT)font )
		font.DeleteObject();

	return font.CreateFontIndirect(&lf);
}

void CPaulaListPreviewPrint::SetMargins(CDC* pDC, CPrintInfo* pInfo) const
{
	ASSERT(NULL != pInfo);

	CRect rcMargins = s_rcMargins;

	CPoint pt( pDC->GetDeviceCaps (LOGPIXELSX),
						 pDC->GetDeviceCaps (LOGPIXELSY) );

	CSize sizePhysicalOffset(
		pDC->GetDeviceCaps(PHYSICALOFFSETX),
		pDC->GetDeviceCaps(PHYSICALOFFSETY ));

	// Translate to inches

	rcMargins.left =   ::MulDiv( rcMargins.left,   pt.x, 2540 );
	rcMargins.top =    ::MulDiv( rcMargins.top,    pt.y, 2540 );
	rcMargins.right =  ::MulDiv( rcMargins.right,  pt.x, 2540 );
	rcMargins.bottom = ::MulDiv( rcMargins.bottom, pt.y, 2540 );
 	
  LONG xWidth = sizePhysicalOffset.cx + pDC->GetDeviceCaps(HORZRES) - rcMargins.right;
	LONG yHeight = sizePhysicalOffset.cy + pDC->GetDeviceCaps(VERTRES) - rcMargins.bottom ;

	pDC->IntersectClipRect(0, 0, xWidth, yHeight);

  pDC->OffsetViewportOrg (rcMargins.left - sizePhysicalOffset.cx, 
													rcMargins.top - sizePhysicalOffset.cy );

	pInfo->m_rectDraw.left = 0;
	pInfo->m_rectDraw.top = 0;
	pInfo->m_rectDraw.right = pDC->GetDeviceCaps(PHYSICALWIDTH) - rcMargins.left - rcMargins.right;
	pInfo->m_rectDraw.bottom = pDC->GetDeviceCaps(PHYSICALHEIGHT) - rcMargins.top - rcMargins.bottom;
}

//////////////////////////////////////////////////////////////////////
// Profile settings helpers 

void CPaulaListPreviewPrint::GetProfileFont(LPCTSTR szSec, LOGFONT& lf)
{
	CWinApp* pApp = AfxGetApp();
	ASSERT(NULL != pApp);

	::ZeroMemory(&lf, sizeof(lf));

	lf.lfHeight = pApp->GetProfileInt(szSec, szHeight, 0xFFFFFFF3);
	lf.lfWeight = pApp->GetProfileInt(szSec, szWeight, 400);
	lf.lfItalic = (BYTE)pApp->GetProfileInt(szSec, szItalic, 0);
	lf.lfUnderline = (BYTE)pApp->GetProfileInt(szSec, szUnderline, 0);
	lf.lfPitchAndFamily = (BYTE)pApp->GetProfileInt(szSec, szPitchAndFamily, 0);
	lf.lfCharSet = (BYTE)pApp->GetProfileInt(szSec, szCharSet, DEFAULT_CHARSET);
	CString strFont = pApp->GetProfileString(szSec, szFaceName, _T("Arial"));
	_tcsncpy((TCHAR*)lf.lfFaceName, strFont, sizeof lf.lfFaceName);
}

void CPaulaListPreviewPrint::WriteProfileFont(LPCTSTR szSec, const LOGFONT& lf)
{
	CWinApp* pApp = AfxGetApp();
	ASSERT(NULL != pApp);

	pApp->WriteProfileInt(szSec, szHeight, lf.lfHeight);
	pApp->WriteProfileInt(szSec, szHeight, lf.lfHeight);
	pApp->WriteProfileInt(szSec, szWeight, lf.lfWeight);
	pApp->WriteProfileInt(szSec, szItalic, lf.lfItalic);
	pApp->WriteProfileInt(szSec, szUnderline, lf.lfUnderline);
	pApp->WriteProfileInt(szSec, szPitchAndFamily, lf.lfPitchAndFamily);
	pApp->WriteProfileInt(szSec, szCharSet, lf.lfCharSet);
	pApp->WriteProfileString(szSec, szFaceName, (LPCTSTR)lf.lfFaceName);
}

void CPaulaListPreviewPrint::GetProfileSettings(LPCTSTR szRegKey)
{
	ASSERT( NULL != szRegKey );
	CString strRegKey  = szRegKey;
 
	// default : Arial, 10 
	GetProfileFont(strRegKey + szFont, s_lf);

	// default margin for each side 1 cm
	CWinApp* pApp = AfxGetApp();
	CRect rc;

	rc.left   = pApp->GetProfileInt(strRegKey + szMargin, szLeft, 1000);
	rc.top	  = pApp->GetProfileInt(strRegKey + szMargin, szTop, 1000);
	rc.right  = pApp->GetProfileInt(strRegKey + szMargin, szRight, 1000);
	rc.bottom = pApp->GetProfileInt(strRegKey + szMargin, szBottom, 1000);

	s_rcMargins = rc;
}

void CPaulaListPreviewPrint::WriteProfileSettings(LPCTSTR lpszRegKey)
{
	CString strRegKey = lpszRegKey;

	WriteProfileFont(strRegKey + szFont, s_lf);

	CRect rc = s_rcMargins;
	CWinApp* pApp = AfxGetApp();

	pApp->WriteProfileInt(strRegKey + szMargin, szLeft, rc.left);
	pApp->WriteProfileInt(strRegKey + szMargin, szTop, rc.top);
	pApp->WriteProfileInt(strRegKey + szMargin, szRight, rc.right);
	pApp->WriteProfileInt(strRegKey + szMargin, szBottom, rc.bottom);

}

///////////////////////////////////////////////////////////////
// Fills in the print cache and does runtime pagination
CPrintCache* CPaulaListPreviewPrint::GetDataForThisPage(CPrintInfo* pInfo)
{
	ASSERT( NULL != m_pPrintCache1);
	ASSERT( NULL != m_pPrintCache2);
	ASSERT( NULL != pInfo );

	//
	// Use the data already cached if possible
	//

	if ( pInfo->m_nCurPage ==  m_pPrintCache1->m_nPrintPage )
	{
		m_iLRUCache = 1;
		return m_pPrintCache1;
	}

	if ( pInfo->m_nCurPage ==  m_pPrintCache2->m_nPrintPage )
	{
		m_iLRUCache = 2;
		return m_pPrintCache2;
	}

	//
	// Reqired page is not in the print cache.
	// Get the data from the list control.
	//

	CPrintCache *pPrintCache = NULL;

	if ( 1 != m_iLRUCache )
	{
		pPrintCache = m_pPrintCache1;
		m_iLRUCache = 1;
	}
	else
	{
		pPrintCache = m_pPrintCache2;
		m_iLRUCache = 2;
	}


	pPrintCache->m_nPrintPage = pInfo->m_nCurPage ;

	CListCtrl& listCtrl = GetListCtrl();
	BOOL bRetriveImages = (m_arIcons.GetSize() > 0 );


	LV_ITEM lv;
	::ZeroMemory(&lv, sizeof(lv) );

	TCHAR szText[1024];
	lv.pszText = szText;
	lv.cchTextMax = sizeof(szText)/sizeof(szText[0]);
	lv.mask = LVIF_TEXT | ( bRetriveImages ? LVIF_IMAGE : 0x0 );

	LONG nFirstItem = m_arStartItemForThisPage[pInfo->m_nCurPage];

	LONG nLinesOnThatPage = ( pInfo->m_rectDraw.Height() / GetLineHeight() ) ;
	nLinesOnThatPage -= 2;													// space for the column headers
	nLinesOnThatPage = max ( nLinesOnThatPage, 1 ); // yet at least one item per page

	LONG nLastItem = nFirstItem + nLinesOnThatPage;

	int nColumnCount = m_PrintColumns.GetSize();
	int nIconsCount  = m_arIcons.GetSize();

	// if list control has got OWNERDATA flag, GetItem will never
	// return FALSE (at least in the current version of commctr32.dll).
	// Just empty strings will be put into the cache for the rest
	// of the page
	
	BOOL bNoMoreItems = FALSE;
	for ( int iItem = nFirstItem ; !bNoMoreItems && iItem < nLastItem ; iItem++ )
	{
		for ( int iCol = 0 ; iCol < nColumnCount ; iCol ++ )
		{

			lv.iItem = iItem;
			lv.iSubItem = iCol;
			*szText = _T('\0');
			lv.iImage = -1;

			if( listCtrl.GetItem(&lv) )
			{
				HICON hIcon = ( lv.iImage >= 0 && lv.iImage < nIconsCount ) ? m_arIcons[lv.iImage] : NULL ;
				pPrintCache->Put(iItem - nFirstItem, iCol, szText, hIcon);
			}
			else
			{	// No more items
				bNoMoreItems = TRUE;
				break;
			}
		}
	}

	pPrintCache->m_nValidLines = iItem - nFirstItem - ( bNoMoreItems ? 1 : 0 );


	//
	// Runtime pagination
	//

	if ( listCtrl.GetItemCount() > iItem )
	{
		m_arStartItemForThisPage.SetAtGrow( pInfo->m_nCurPage + 1, iItem);
	}

	pInfo->SetMaxPage( m_arStartItemForThisPage.GetSize() - 1);

	ASSERT( NULL != pPrintCache);
	return 	pPrintCache;
}	


BEGIN_MESSAGE_MAP(CPaulaListPreviewPrint, CView)
	//{{AFX_MSG_MAP(CPaulaListPreviewPrint)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CPaulaListPreviewPrint::OnDraw(CDC* pDC)
{
	// No drawing
}


/////////////////////////////////////////////////////////////////////////////
// CPaulaListPreviewPrint diagnostics

#ifdef _DEBUG
void CPaulaListPreviewPrint::AssertValid() const
{
	CView::AssertValid();
}

void CPaulaListPreviewPrint::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPaulaListPreviewPrint printing

void CPaulaListPreviewPrint::OnPrintHeader(CDC* pDC, CPrintInfo* pInfo)
{
	ASSERT(pDC);
	ASSERT(pInfo);

	LONG cx = pInfo->m_rectDraw.left;
	LONG cy = pInfo->m_rectDraw.top;
	
	CString strOut = _T("FrankenDataWareHouse");

	pDC->TextOut(cx, cy, strOut);

	cy += (LONG)(GetLineHeight() * 1.5);

	pDC->MoveTo( cx, cy );
	pDC->LineTo( pInfo->m_rectDraw.Width(), cy);

	cy += (LONG)(GetLineHeight() * 0.5);

	pInfo->m_rectDraw.top += 2 * GetLineHeight();

	// draw header
	if ( !m_strHeader.IsEmpty() )
	{
		CRect rc = pInfo->m_rectDraw;
		UINT uFlags = DT_LEFT | DT_TOP | DT_WORDBREAK |
									DT_EXPANDTABS | DT_NOPREFIX | DT_TABSTOP;

		pInfo->m_rectDraw.top+= pDC->DrawText(m_strHeader, rc, uFlags);
		pInfo->m_rectDraw.top += GetLineHeight();
	}
}

void CPaulaListPreviewPrint::OnPrintFooter(CDC* pDC, CPrintInfo* pInfo)
{
	ASSERT(pDC);
	ASSERT(pInfo);

	//
	// _________________________________________________________
	// 
	// Page #                 FrankenDataWareHouse, <Timestamp>

	pInfo->m_rectDraw.bottom -= 2 * GetLineHeight();
	
	LONG cx = pInfo->m_rectDraw.left;
	LONG cy = pInfo->m_rectDraw.bottom;

	cy += (LONG)(GetLineHeight() * 0.5);

	pDC->MoveTo( cx, cy );
	pDC->LineTo( pInfo->m_rectDraw.Width(), cy);

	cy += (LONG)(GetLineHeight() * 0.5);


	CString strPageFormat = 	// returns "Page %u"
			pInfo->m_strPageDesc.Left(pInfo->m_strPageDesc.Find(_T("\n")));

	CString strOut;

	strOut.Format( strPageFormat, pInfo->m_nCurPage);
	pDC->TextOut(cx, cy, strOut);

	CRect rc( cx, cy, pInfo->m_rectDraw.Width(), cy +  GetLineHeight());
	strOut = m_dateNow.Format();

	pDC->DrawText(strOut, rc, DT_SINGLELINE | DT_NOCLIP | DT_NOPREFIX | DT_RIGHT);

}

void CPaulaListPreviewPrint::OnPrintBody(CDC* pDC, CPrintInfo* pInfo)
{
	ASSERT(pDC);
	ASSERT(pInfo);

	CPrintCache* pPrintCache = GetDataForThisPage(pInfo);
	ASSERT(pPrintCache);

	LONG cx = pInfo->m_rectDraw.left;
	LONG cy = pInfo->m_rectDraw.top;

	int nColumns = m_PrintColumns.GetSize(); 

	// Deflate the item rectangle by that value to make
	// some spacing between the columns 

	LONG xDeflateItemRect = (LONG)( GetLineHeight() * 0.25 );

	//
	// Draw column captions
	//

	//
	// Column 1    | Column2    |...
	//

	for ( int iCol = 0 ; iCol < nColumns ; iCol ++ )
	{
		CPrintColumn pc = m_PrintColumns[iCol];
		LPCTSTR lpszText = pc.m_strCaption;

		CRect rc(cx, cy, cx + pc.m_iWidth, cy + GetLineHeight());
		rc.DeflateRect(xDeflateItemRect, 0);

		UINT uFormat = DT_SINGLELINE | DT_NOCLIP | DT_NOPREFIX | pc.m_iFormat ;

		pDC->DrawText(lpszText, -1, rc, uFormat);

		cx += pc.m_iWidth ;
	}

	//
	// Underline column captions
	// _______________________________ ...

	cy += (int)(GetLineHeight() * 1.5);
	cx = pInfo->m_rectDraw.left;

	pDC->MoveTo( cx, cy );
	pDC->LineTo( pInfo->m_rectDraw.Width(), cy);

	cy += (int)(GetLineHeight() * 0.5);

	//
	// Draw column data
	//

	for ( int iRow = 0 ; iRow < pPrintCache->GetValidLines() ; iRow++ )
	{
		for ( cx = pInfo->m_rectDraw.left, iCol = 0 ; iCol < nColumns ; iCol ++ )
		{
				
			CPrintColumn pc = m_PrintColumns[iCol];
			CPrintCacheEntry& data = pPrintCache->Get(iRow, pc.m_iOrigin);

			CRect rc(cx, cy, cx + pc.m_iWidth, cy + GetLineHeight());
			rc.DeflateRect( xDeflateItemRect, 0);

			if ( pc.m_bHasImages || NULL != data.m_hIcon )
			{ 
				if( NULL != data.m_hIcon  )
				{
					VERIFY( ::DrawIconEx( *pDC, rc.left, rc.top, data.m_hIcon, rc.Height(), rc.Height(), 0, NULL, DI_NORMAL ));
				}
				rc.left += rc.Height() + xDeflateItemRect;
			}
	
			if ( !data.m_strText.IsEmpty() )
			{
				UINT uFormat = DT_SINGLELINE | DT_NOCLIP | DT_NOPREFIX | pc.m_iFormat ;
				pDC->DrawText(data.m_strText, -1, rc, uFormat);
			}
			cx += pc.m_iWidth ;
		}

		cy += GetLineHeight();
	}

}

void CPaulaListPreviewPrint::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
	ASSERT_VALID(this);
	CWaitCursor wait;

	m_dateNow = COleDateTime::GetCurrentTime();

	CListCtrl& listCtrl = GetListCtrl();
	
	//
	// Create the desired font and select it into the
	// device context to make the page layout calculations
	//

	CreatePrintFont(pDC, m_Font, s_lf);
	CFont* pFontOld = pDC->SelectObject(&m_Font);

	VERIFY( pDC->GetTextMetrics(&m_tm) );

	//
	// Restore the font
	//

	pDC->SelectObject(pFontOld);

	//
	// Get the metrics of control's font to map the control
	// column width to the print column width.
	//

	TEXTMETRIC tmCtrl;
 
	CDC *pCtrlDC = listCtrl.GetWindowDC();	
	ASSERT(pCtrlDC);

	if ( pCtrlDC )
	{
		CFont *pCtrlFont = listCtrl.GetFont();
		CFont *pCtrlFontOld = pCtrlDC->SelectObject(pCtrlFont);

		VERIFY( pCtrlDC->GetTextMetrics(&tmCtrl) );

		pCtrlFontOld ? pCtrlDC->SelectObject(pCtrlFontOld) : NULL;

		//
		// 	Release control DC
		//

		listCtrl.ReleaseDC(pCtrlDC);
	}
	else
	{
		ASSERT(FALSE);
	}


	//
	// Try to get the image list
	//

	CImageList*  pImageList = listCtrl.GetImageList(LVSIL_SMALL);

	if ( pImageList )
	{
		int nCount = pImageList->GetImageCount();
		m_arIcons.SetSize(nCount);

		for ( int i = 0 ; i < nCount ; i++ )
		{
			m_arIcons[i] = pImageList->ExtractIcon(i);
		}
	}

	//
	// Control's columns
	//

	int nColumnCount = GetListColumnCount();	
	m_PrintColumns.SetSize(nColumnCount);

	for( int nCol = 0 ; nCol < nColumnCount ; nCol++ )
	{
		LV_COLUMN lc;
		TCHAR szText[1024];
		::ZeroMemory( &lc, sizeof(lc) );

		CPrintColumn pc;

		lc.pszText = szText;
		lc.cchTextMax = sizeof(szText);
		lc.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH | LVCF_ORDER ;

		VERIFY( listCtrl.GetColumn( nCol, &lc ) );

		pc.m_strCaption = lc.pszText;
		
		switch ( lc.fmt & ( LVCFMT_LEFT | LVCFMT_RIGHT | LVCFMT_CENTER ) )
		{
			default : ASSERT(FALSE);
			case LVCFMT_LEFT   : pc.m_iFormat = DT_LEFT;  break;
			case LVCFMT_RIGHT  : pc.m_iFormat = DT_RIGHT; break;
			case LVCFMT_CENTER : pc.m_iFormat = DT_CENTER;; break;
		}

		pc.m_iWidth = ::MulDiv( lc.cx, m_tm.tmAveCharWidth,	tmCtrl.tmAveCharWidth) ;	

		if ( NULL != pImageList )
		{
			// The first column always has images; however, if
			// LVS_EX_SUBITEMIMAGES flag is set, the column might
			// have images even if m_bHasImages is FALSE.

			pc.m_bHasImages = (0 == nCol);
		}		

		// The columns are stored in the order of their appearance

		pc.m_iOrigin = nCol;
		m_PrintColumns[lc.iOrder] = pc;
	}

	/////////////////////////////////////////////////////
	// Prepare printing caches. We'll need two of them
	// ( for two-pages preview ).

	ASSERT( NULL == m_pPrintCache1 && NULL == m_pPrintCache2 );

	// At this point we do not know anything about the page 
	// layout. However the page body height should not
	// exceed the GetDeviceCaps(VERTRES). Divided by the line 
	// height it yelds max size for the cache.

	LONG lMaxLinesPerPage = pDC->GetDeviceCaps(VERTRES) / GetLineHeight();
	lMaxLinesPerPage = max( lMaxLinesPerPage, 1 );

	m_pPrintCache1 = new CPrintCache( lMaxLinesPerPage, nColumnCount );
	m_pPrintCache2 = new CPrintCache( lMaxLinesPerPage, nColumnCount );

	m_arStartItemForThisPage.SetSize(2, 10);
	m_arStartItemForThisPage[1] = 0;
}

void CPaulaListPreviewPrint::OnPrintCoverPage(CDC* pDC, CPrintInfo* pInfo)
{
	ASSERT_VALID(this);
	ASSERT(NULL != pDC);
	ASSERT(NULL != pInfo);

	// draw cover page text if not empty

	CRect rc = pInfo->m_rectDraw;
	UINT uFlags = DT_LEFT | DT_TOP | DT_WORDBREAK |
								DT_EXPANDTABS | DT_NOPREFIX | DT_TABSTOP;

	pInfo->m_rectDraw.top += pDC->DrawText(m_strCoverPage, rc, uFlags);
	pInfo->m_rectDraw.top += GetLineHeight();

}

void CPaulaListPreviewPrint::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	ASSERT_VALID(this);
	ASSERT(NULL != pDC);
	ASSERT(NULL != pInfo);

	int iSave = pDC->SaveDC();

	SetMargins(pDC, pInfo);

	CFont* pFontOld = pDC->SelectObject(&m_Font);

	OnPrintHeader(pDC, pInfo);
	OnPrintFooter(pDC, pInfo);

	if ( 1 == pInfo->m_nCurPage && !m_strCoverPage.IsEmpty() )
	{
		OnPrintCoverPage(pDC, pInfo);

		// correct the number of pages
		m_arStartItemForThisPage.SetAtGrow(2,0);
	}
	else
	{
		OnPrintBody(pDC, pInfo);
	}
	
	pDC->SelectObject(pFontOld);

	pDC->RestoreDC(iSave);

}

void CPaulaListPreviewPrint::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
	ASSERT_VALID(this);
	CleanUp();	
}

void CPaulaListPreviewPrint::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	ASSERT_VALID(this);
	ASSERT(pInfo != FALSE);

	if ( 0xffff != pInfo->GetMaxPage() )
	{
		if ( pInfo->m_nCurPage  > pInfo->GetMaxPage() )
			pInfo->m_bContinuePrinting = FALSE;
	}
}

BOOL CPaulaListPreviewPrint::OnPreparePrinting(CPrintInfo* pInfo) 
{
	ASSERT_VALID(this);
	return DoPreparePrinting(pInfo);
}


void CPaulaListPreviewPrint::DoPageSetupDialog()
{
	CWaitCursor wait;
	
	CWinApp* pApp = AfxGetApp();

	PRINTDLG pd;
	ZeroMemory(&pd, sizeof (pd) );

	if ( ! pApp->GetPrinterDeviceDefaults(&pd))
	{
		return;
	}

	// WORKAROUND : Page Setup Dialog does not prevent the user
	// from entering zero margins when PSD_INHUNDREDTHSOFMILLIMETERS
	// is specified, so we got to set them by ourselfs

	CDC dcPrn;
	if ( ! pApp->CreatePrinterDC( dcPrn ) )
	{
		return;
	}

	CRect rcMinMargins;
	rcMinMargins.left = dcPrn.GetDeviceCaps(PHYSICALOFFSETX);
	rcMinMargins.top = dcPrn.GetDeviceCaps(PHYSICALOFFSETY);
	rcMinMargins.right = dcPrn.GetDeviceCaps(PHYSICALWIDTH) - dcPrn.GetDeviceCaps(HORZRES) - rcMinMargins.left;
	rcMinMargins.bottom = dcPrn.GetDeviceCaps(PHYSICALHEIGHT) - dcPrn.GetDeviceCaps(VERTRES) - rcMinMargins.top;

	// Translate to 0.01 of mm

	rcMinMargins.left = ::MulDiv ( rcMinMargins.left, 2540, dcPrn.GetDeviceCaps(LOGPIXELSX));
	rcMinMargins.top = ::MulDiv ( rcMinMargins.top, 2540, dcPrn.GetDeviceCaps(LOGPIXELSY));
	rcMinMargins.right = ::MulDiv ( rcMinMargins.right, 2540, dcPrn.GetDeviceCaps(LOGPIXELSX));
	rcMinMargins.bottom = ::MulDiv ( rcMinMargins.bottom, 2540, dcPrn.GetDeviceCaps(LOGPIXELSY));

	dcPrn.DeleteDC();

	// End workaround ///////////////

	CPageSetupDialog dlgPageSetup;	

	dlgPageSetup.m_psd.rtMargin = s_rcMargins;
	dlgPageSetup.m_psd.rtMinMargin = rcMinMargins;
	dlgPageSetup.m_psd.Flags |= PSD_INHUNDREDTHSOFMILLIMETERS | PSD_MINMARGINS | PSD_MARGINS;
	dlgPageSetup.m_psd.hDevNames = pd.hDevNames;
	dlgPageSetup.m_psd.hDevMode = pd.hDevMode;

	if ( IDOK == dlgPageSetup.DoModal() )
	{ 
		pApp->SelectPrinter( dlgPageSetup.m_psd.hDevNames	, 
												 dlgPageSetup.m_psd.hDevMode );
		
		s_rcMargins = dlgPageSetup.m_psd.rtMargin;
	}
}


//////////////////////////////////////////////////////////////////////////////
// The ywo functions below, ShowPrintPreview() and virtual OnEndPrintPreview()
// were added in order to be able to handle the preview from a modeless dialog.
//
// ShowPrintPreview() does the same job as CView::OnFilePrintPreview() except
// that a pointer to CPrintPreviewState allocated structure is saved.
// The reason for doing so is explained in OnEndPrintPreview.
// 
// OnEndPrintPreview distinguishes two cases: first, if the preview was done 
// in a 'normal' way, i.e. from some other CListView class, it simply calls the
// base class implementation for OnEndPrintPreview() ( just as MFC 4.2 documentation
// recomends). 
// 
// Trying to do the same when ShowPrintPreview() function was called
// from a modeless dialog (with no actual view-window) leads to a situation 
// where the current instance of CPaulaListPreviewPrint, being set by the framework 
// as active view starts getting numerous messages after the previewing is done,
// which ends up with a crash.
//
// So we do just the same job as CView::OnEndPrintPreview, covinced that
// pView->m_pPrintView is 'this' object, and avoid any interactions with
// pseudo-view.

void CPaulaListPreviewPrint::ShowPrintPreviewFromDialog(const CDialog* pDlg, CListCtrl* pListCtrl)
{ 
	ASSERT_VALID(this);
	ASSERT_KINDOF(CDialog, pDlg);
	ASSERT_KINDOF(CListCtrl, pListCtrl);

	SetPrintSourceCtrl(*pListCtrl);
	
	CPrintPreviewState* pState = new CPrintPreviewState;
	m_pPreviewState = pState;
	m_hWndModelessDlg = pDlg->GetSafeHwnd();

	if (!DoPrintPreview(AFX_IDD_PREVIEW_TOOLBAR, this,
							RUNTIME_CLASS(CPreviewView), pState))
	{
		// In derived classes, reverse special window handling here for
		// Preview failure case

		TRACE0("Error: DoPrintPreview failed.\n");
		AfxMessageBox(AFX_IDP_COMMAND_FAILURE);
		m_pPreviewState = NULL;
		delete pState;      // preview failed to initialize, delete State now
	}
}

//
// See the comment above.
//
void CPaulaListPreviewPrint::OnEndPrintPreview(CDC* pDC, CPrintInfo* pInfo, POINT point, CPreviewView* pView) 
{
	ASSERT_VALID(pDC);
	ASSERT_VALID(pView);

	// Preview was initiated from a modeless dialog via 
	// ShowPrintPreview()

	if ( NULL == m_pPreviewState )
	{
		// The usual way
		CView::OnEndPrintPreview(pDC, pInfo, point, pView);
		return;
	}
	// else : this == pView->m_pPrintView 

	OnEndPrinting(pDC, pInfo);

	CFrameWnd* pParent = (CFrameWnd*)AfxGetThread()->m_pMainWnd;
		
	ASSERT_VALID(pParent);
	ASSERT_KINDOF(CFrameWnd, pParent);

	// restore the old main window
	pParent->OnSetPreviewMode(FALSE, m_pPreviewState);

	// m_pPreviewState will be deleted by CPreviewView
	m_pPreviewState = NULL;

	// Was not called from a view, so set active view to NULL
	pParent->SetActiveView(NULL);

	pView->DestroyWindow(); // destroy preview view
													// C++ object will be deleted in PostNcDestroy

	// restore main frame layout and idle message
	pParent->RecalcLayout();
	pParent->SendMessage(WM_SETMESSAGESTRING, (WPARAM)AFX_IDS_IDLEMESSAGE, 0L);
	pParent->UpdateWindow();	

	if  ( NULL != m_hWndModelessDlg )
		::SetFocus(m_hWndModelessDlg);

	m_hWndModelessDlg = NULL;
}

void CPaulaListPreviewPrint::DoPrintFromDialog(const CDialog* pDlg, CListCtrl* pListCtrl)
{
	ASSERT_VALID(this);
	ASSERT_KINDOF(CDialog, pDlg);
	ASSERT_KINDOF(CListCtrl, pListCtrl);

	SetPrintSourceCtrl(*pListCtrl);
	OnFilePrint();
}

