#if !defined(AFX_PAULALISTPREVIEWPRINT_H__95352CA1_EE61_11D1_959C_00609713D1D0__INCLUDED_)
#define AFX_PAULALISTPREVIEWPRINT_H__95352CA1_EE61_11D1_959C_00609713D1D0__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
// PaulaListPreviewPrint.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CPaulaListPreviewPrint class
//
// To use CPaulaListPreviewPrint from a CListView class, simply
// include DECLARE_PRINTABLE( <your view class name> ) macro
// into you header file and IMPLEMENT_PRINTABLE( <your view class name> )
// macro into your implementation .cpp file.
//
// To use it from a modeless dialog, declare a member variable of type 
// CPaulaListPreviewPrint in your dialog and call ShowPrintPreviewFromDialog() /
// DoPrintFromDialog() to preview/print.

class CPrintCache;	// simple helper

class CPaulaListPreviewPrint : public CView
{
public:
	CPaulaListPreviewPrint();
	virtual ~CPaulaListPreviewPrint();

	DECLARE_DYNAMIC(CPaulaListPreviewPrint)
	
// Attributes
public:

	static LOGFONT s_lf;					// font used for printing
	static CRect s_rcMargins;		// in hundredths of millimeters

// Operations
public:

	// Settings

	static void DoPageSetupDialog();
	static void DoFontDialog();

	// You must call this member funtion
	// priore to using CPaulaListPreviewPrint 
	
	void SetPrintSourceCtrl(CListCtrl& listCtrl);
	void SetHeaderText(LPCTSTR lpszHeader);						// string to be printed in the header
	void SetCoverPageText(LPCTSTR lpszCoverPageText);	// Text to be printed on the first page

	// Call those members to preview/print a list control 
	// from a modeless dialog. DO NOT call them from a view!

	void ShowPrintPreviewFromDialog(const CDialog* pDlg, CListCtrl* pListCtrl);
	void DoPrintFromDialog(const CDialog* pDlg, CListCtrl* pListCtrl);
	
	// Profile settings

	static void GetProfileSettings(LPCTSTR szRegKey);
	static void WriteProfileSettings(LPCTSTR szRegKey);	

	static void WriteProfileFont(LPCTSTR szSec, const LOGFONT& lf);
	static void GetProfileFont(LPCTSTR szSec, LOGFONT& lf);

	//
	// All printig member function must be public
	// in order to allow the redirection
	//

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CPaulaListPreviewPrint)
	public:
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
	protected:
	virtual void OnEndPrintPreview(CDC* pDC, CPrintInfo* pInfo, POINT point, CPreviewView* pView);
	//}}AFX_VIRTUAL

// Implementation
protected:
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	HWND m_hWndListCtrl;							// Source list control
	
	// CPrintCache is a simple helper class which basicly
	// implements a two-dimensional dynamic-size array	

	CPrintCache*	m_pPrintCache1;
	CPrintCache*	m_pPrintCache2;
	UINT m_iLRUCache;

	// The first item to be printed on the corresponding page, i.e 
	// for page 1 m_arStartItemForThisPage[1] = 0, 
	// for page 2 m_arStartItemForThisPage[2] will be something like 60 etc.

	CDWordArray m_arStartItemForThisPage;	
	COleDateTime m_dateNow;						// timestamp in the footer
	CFont m_Font;											// font created from s_lf
	TEXTMETRIC m_tm;									// text metrics of m_Font 
	CArray<HICON, HICON> m_arIcons;		// icons extracted from the list control

	CString m_strHeader;							// string to be printed as header on every page
	CString m_strCoverPage;						// string to be printed as header on the first page

	// simple helper class ; desctribes a column to be printed
	class CPrintColumn
	{
	public:
		CPrintColumn()
		{
			m_iFormat = DT_LEFT;
			m_iWidth = 0;
			m_bHasImages = FALSE;
			m_iOrigin = 0;				
		}

		CString m_strCaption;		// Column caption
		UINT m_iFormat;					// DT_LEFT, DT_RIGHT, DT_CENTER
		int m_iWidth;						// Width of the column in LU
		BOOL m_bHasImages;			// TRUE if the column has images
		int m_iOrigin;					// Column number
	};

	// The array contains the columns in order of their
	// appearance.  

	CArray<CPrintColumn, CPrintColumn&> m_PrintColumns;

	//
	// For modeless dialogs
	//

	CPrintPreviewState* m_pPreviewState;	
	HWND m_hWndModelessDlg;	// To set back the focus after preview
	
protected:

	void CleanUp();
	BOOL CreatePrintFont(CDC* pDC, CFont& font, LOGFONT logFont);
	void SetMargins(CDC* pDC, CPrintInfo* pInfo) const;
	CListCtrl& GetListCtrl();
	int GetListColumnCount();
	CPrintCache* GetDataForThisPage(CPrintInfo* pInfo);

	LONG GetLineHeight() const;

	virtual void OnPrintCoverPage(CDC* pDC, CPrintInfo* pInfo); 
	virtual void OnPrintBody(CDC* pDC, CPrintInfo* pInfo); 

	// Must deflate pInfo->m_rectDraw by the size of the footer/header
	// drawing area.

	virtual void OnPrintHeader(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnPrintFooter(CDC* pDC, CPrintInfo* pInfo);

	// Generated message map functions
protected:
	//{{AFX_MSG(CPaulaListPreviewPrint)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

inline LONG CPaulaListPreviewPrint::GetLineHeight() const
	{ return m_tm.tmHeight + m_tm.tmExternalLeading; };

///////////////////////////////////////////////////////////////////////////////
//

#define DECLARE_PRINTABLE(class)                                              \
protected:                                                                    \
	CPaulaListPreviewPrint m_print##class;                                      \
public:                                                                       \
virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL)                  \
	{ ASSERT_VALID(this); m_print##class.OnPrepareDC(pDC, pInfo); }             \
protected:                                                                    \
virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo)                       \
	{ ASSERT_VALID(this); m_print##class.OnEndPrinting(pDC, pInfo); }           \
virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)                     \
	{ ASSERT_VALID(this);                                                       \
		m_print##class.SetPrintSourceCtrl( GetListCtrl() );                       \
		m_print##class.OnBeginPrinting(pDC, pInfo);                               \
	}                                                                           \
virtual BOOL OnPreparePrinting(CPrintInfo* pInfo)                             \
	{ ASSERT_VALID(this); return m_print##class.OnPreparePrinting(pInfo); }     \
virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo)                             \
	{ ASSERT_VALID(this); m_print##class.OnPrint(pDC, pInfo); }                 \
public:                                                                       \
void SetHeaderText(LPCTSTR lpszHeader)                                        \
	{ ASSERT_VALID(this); m_print##class.SetHeaderText(lpszHeader); }           \
void SetCoverPageText(LPCTSTR lpszText)                                       \
	{ ASSERT_VALID(this); m_print##class.SetCoverPageText(lpszText); }


#define IMPLEMENT_PRINTABLE(class)

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_PAULALISTPREVIEWPRINT_H__95352CA1_EE61_11D1_959C_00609713D1D0__INCLUDED_)
